@@ -1,9 +1,9 @@
 /* deets: luau.c
-   Copyright (C) 2009, 2010  Clint Adams
+   Copyright (C) 2009, 2010, 2011  Clint Adams
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,8 +18,8 @@
 
 #include <stdio.h>
 
-#include <setjmp.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -68,7 +68,7 @@ dpkg_status (lua_State * L)
 	  lua_error (L);
 	}
 
-      p = findpackage (lua_tostring (L, i));
+      p = pkg_db_find (lua_tostring (L, i));
       switch (p->status)
 	{
 	case stat_notinstalled:
@@ -112,12 +112,13 @@ dpkg_pkglist (lua_State * L)
 
   lua_newtable (L);
 
-  for (iter = iterpkgstart (); (pkg = iterpkgnext (iter));)
+  for (iter = pkg_db_iter_new (); (pkg = pkg_db_iter_next (iter));)
     {
       lua_pushnumber (L, ++i);
       lua_pushstring (L, pkg->name);
       lua_rawset (L, -3);
     }
+  pkg_db_iter_free (iter);
 
   return 1;
 }
@@ -136,7 +137,7 @@ dpkg_conffilelist (lua_State * L)
       lua_error (L);
     }
 
-  p = findpackage (lua_tostring (L, 1));
+  p = pkg_db_find (lua_tostring (L, 1));
   if (!(c = p->installed.conffiles))
     {
       lua_pushnil (L);
@@ -240,7 +241,7 @@ deets_tempstat (lua_State * L)
 	  else
 	    lua_pushstring (L, "other");
 
-	  snprintf (&m, 24, "%0o", s.st_mode & 007777);
+	  snprintf (m, 24, "%0o", s.st_mode & 007777);
 	  lua_pushstring (L, m);
 	  lua_pushnumber (L, s.st_uid);
 	  lua_pushnumber (L, s.st_gid);
@@ -276,38 +277,38 @@ populate_systeminfo (lua_State * L)
   lua_getfield (L, LUA_GLOBALSINDEX, "deets");
   lua_newtable (L);
 
-  i = gethostname ((char *) &buf, 63);
+  i = gethostname (buf, 63);
   lua_pushstring (L, "hostname");
   if (i)
     lua_pushnil (L);
   else
     {
       buf[63] = '\0';
-      lua_pushstring (L, (const char *) &buf);
+      lua_pushstring (L, buf);
       memset (&hints, 0, sizeof (struct addrinfo));
       hints.ai_socktype = SOCK_DGRAM;
       hints.ai_flags = AI_CANONNAME;
 
-      if (getaddrinfo (&buf, NULL, &hints, &carne_de))
+      if (getaddrinfo (buf, NULL, &hints, &carne_de))
 	carne_de = NULL;
     }
   lua_rawset (L, -3);
 
   lua_pushstring (L, "fqdn");
   if (carne_de && carne_de->ai_canonname)
-    lua_pushstring (L, (const char *) carne_de->ai_canonname);
+    lua_pushstring (L, carne_de->ai_canonname);
   else
     lua_pushnil (L);
   lua_rawset (L, -3);
 
-  i = getdomainname ((char *) &buf, 63);
+  i = getdomainname (buf, 63);
   lua_pushstring (L, "nis_domainname");
   if (i)
     lua_pushnil (L);
   else
     {
       buf[63] = '\0';
-      lua_pushstring (L, (const char *) &buf);
+      lua_pushstring (L, buf);
     }
   lua_rawset (L, -3);
 
@@ -358,32 +359,21 @@ main (int argc, char **argv)
   int c;
   int digit_optind = 0;
 
-  jmp_buf ejbuf;
-
   int status;
 
   char *tempdir;
   char template_dir[] = "/tmp/deets.XXXXXX";	/* use TEMPDIR instead? */
 
   fprintf (stderr, "deets luau version " PACKAGE_VERSION
-	  "\nCopyright (C) 2009, 2010  Clint Adams\n");
+	   "\nCopyright (C) 2009, 2010, 2011  Clint Adams\n");
   fprintf (stderr, "deets comes with ABSOLUTELY NO WARRANTY.\n");
-  fprintf (stderr, "This is free software, and you are welcome to redistribute it\n");
+  fprintf (stderr,
+	   "This is free software, and you are welcome to redistribute it\n");
   fprintf (stderr, "under certain conditions.\n");
 
-  do
-    {
-      if (setjmp (ejbuf))
-	{
-	  error_unwind (ehflag_bombout);
-	  exit (2);
-	}
-      push_error_handler (&ejbuf, print_error_fatal, NULL);
-    }
-  while (0);
+  push_error_context ();
 
-
-  modstatdb_init ("/var/lib/dpkg", msdbrw_readonly | msdbrw_noavail);
+  modstatdb_open (msdbrw_readonly); /* Use default dpkg db directory. */
 
   while (1)
     {
@@ -453,10 +443,6 @@ main (int argc, char **argv)
   luaL_register (l, "dpkg", dpkglib);
   luaL_register (l, "deets", deetslib);
 
-  /* is it lamer to do the setenv() or the chdir() ? */
-  if (chdir (DEETS_LUADIR))
-    exit (1);
-
   lua_pushboolean (l, audit_mode);
   lua_setglobal (l, "audit_mode");
   lua_pushboolean (l, rectify_mode);
@@ -477,7 +463,7 @@ main (int argc, char **argv)
 
 /*	status = (luaL_loadfile(l, "init.lua") || lua_pcall(l, 0, 0, 0)); */
   status = 999;
-  status = (luaL_loadfile (l, "init.lua"));
+  status = (luaL_loadfile (l, DEETS_LUADIR "/init.lua"));
   lua_call (l, 0, 0);
   if (optind < argc)
     {
@@ -489,13 +475,15 @@ main (int argc, char **argv)
     }
   else
     {
-      status = (luaL_loadfile (l, "default.lua"));
+      status = (luaL_loadfile (l, DEETS_LUADIR "/default.lua"));
       lua_call (l, 0, 0);
     }
-  status = (luaL_loadfile (l, "fin.lua"));
+  status = (luaL_loadfile (l, DEETS_LUADIR "/fin.lua"));
   lua_call (l, 0, 0);
 
   lua_close (l);
 
+  pop_error_context (ehflag_normaltidy);
+
   return 0;
 }
